@@ -1,6 +1,6 @@
 /* The RELP (reliable event logging protocol) core protocol library.
  *
- * Copyright 2008-2016 by Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2008-2018 by Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of librelp.
  *
@@ -33,6 +33,16 @@
 #ifndef RELP_H_INCLUDED
 #define	RELP_H_INCLUDED
 
+#if defined(__GNUC__)
+	#define ATTR_NORETURN __attribute__ ((noreturn))
+	#define ATTR_UNUSED __attribute__((unused))
+	#define ATTR_NONNULL(...) __attribute__((nonnull(__VA_ARGS__)))
+#else
+	#define ATTR_NORETURN
+	#define ATTR_UNUSED
+	#define ATTR_NONNULL(...)
+#endif
+
 #include <pthread.h>
 #if HAVE_SYS_EPOLL_H
 #	include <sys/epoll.h>
@@ -182,6 +192,9 @@ struct relpEngine_s {
 #ifndef RELP_DFLT_WINDOW_SIZE
 #	define RELP_DFLT_WINDOW_SIZE 128 /* 128 unacked frames should be fairly good in most cases */
 #endif
+#ifndef RELP_DFLT_OVERSIZE_MODE
+#	define RELP_DFLT_OVERSIZE_MODE RELP_OVERSIZE_ABORT
+#endif
 
 /* set the default receive buffer size if none is externally configured
  * NOTE: do not define to less than 1.5K or you'll probably see a severe
@@ -221,5 +234,6 @@ static inline int relpEngineShouldStop(relpEngine_t *pThis) {
 relpRetVal relpEngineDispatchFrame(relpEngine_t *pThis, relpSess_t *pSess, relpFrame_t *pFrame);
 void __attribute__((format(printf, 4, 5))) relpEngineCallOnGenericErr(relpEngine_t *pThis,
 	char *eobj, relpRetVal ecode, char *fmt, ...);
+const char * _relpEngine_strerror_r(const int errnum, char *buf, const size_t buflen);
 
 #endif /* #ifndef RELP_H_INCLUDED */
