@@ -39,6 +39,8 @@
  * development.
  */
 #include "config.h"
+#include <stdio.h>
+#include <stdarg.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <assert.h>
@@ -49,6 +51,7 @@
 #include "relp.h"
 #include "relpsess.h"
 #include "relpframe.h"
+#include "relpclt.h"
 #include "sendq.h"
 #include "offers.h"
 #include "dbllinklist.h"
@@ -58,6 +61,35 @@ static relpRetVal relpSessCltDoDisconnect(relpSess_t *pThis);
 static relpRetVal relpSessFixCmdStates(relpSess_t *pThis);
 static relpRetVal relpSessSrvDoDisconnect(relpSess_t *pThis);
 
+/* helper to call onErr if set */
+static void
+callOnErr(const relpSess_t *__restrict__ const pThis,
+	char *__restrict__ const emsg,
+	const relpRetVal ecode)
+{
+	char objinfo[1024];
+	relpTcp_t *const pTcp = pThis->pTcp;
+	//pThis->pEngine->dbgprint("librelp: generic error: ecode %d, "
+		//"emsg '%s'\n", ecode, emsg);
+	if(pThis->pEngine->onErr != NULL) {
+		if(pTcp->pSrv == NULL) { /* client */
+			snprintf(objinfo, sizeof(objinfo), "conn to srvr %s:%s",
+				 pTcp->pClt->pSess->srvAddr,
+				 pTcp->pClt->pSess->srvPort);
+		} else if(pTcp->pRemHostIP == NULL) { /* server listener */
+			snprintf(objinfo, sizeof(objinfo), "lstn %s",
+				 pTcp->pSrv->pLstnPort);
+		} else { /* server connection to client */
+			snprintf(objinfo, sizeof(objinfo), "lstn %s: conn to clt %s/%s",
+				 pTcp->pSrv->pLstnPort, pTcp->pRemHostIP,
+				 pTcp->pRemHostName);
+		}
+		objinfo[sizeof(objinfo)-1] = '\0';
+		pThis->pEngine->onErr(pThis->pUsr, objinfo, emsg, ecode);
+	}
+}
+
+
 /* helper to free permittedPeer structure */
 static inline void
 relpSessFreePermittedPeers(relpSess_t *pThis)
@@ -73,7 +105,7 @@ relpSessFreePermittedPeers(relpSess_t *pThis)
  *  the pSrv parameter may be set to NULL if the session object is for a client.
  */
 relpRetVal
-relpSessConstruct(relpSess_t **ppThis, relpEngine_t *pEngine, int connType, void *pParent)
+relpSessConstruct(relpSess_t **ppThis, relpEngine_t *pEngine, int connType, void *pParent, void *pUsr)
 {
 	relpSess_t *pThis;
 
@@ -86,6 +118,7 @@ relpSessConstruct(relpSess_t **ppThis, relpEngine_t *pEngine, int connType, void
 	}
 
 	RELP_CORE_CONSTRUCTOR(pThis, Sess);
+	pThis->pUsr = pUsr;
 	pThis->pEngine = pEngine;
 	/* use Engine's command enablement states as default */
 	pThis->stateCmdSyslog = pEngine->stateCmdSyslog;
@@ -96,7 +129,6 @@ relpSessConstruct(relpSess_t **ppThis, relpEngine_t *pEngine, int connType, void
 	}
 	pThis->txnr = 1; /* txnr start at 1 according to spec */
 	pThis->timeout = 90;
-	pThis->pUsr = NULL;
 	pThis->sizeWindow = RELP_DFLT_WINDOW_SIZE;
 	pThis->maxDataSize = RELP_DFLT_MAX_DATA_SIZE;
 	pThis->authmode = eRelpAuthMode_None;
@@ -199,7 +231,8 @@ relpSessAcceptAndConstruct(relpSess_t **ppThis, relpSrv_t *pSrv, int sock)
 	RELPOBJ_assert(pSrv, Srv);
 	assert(sock >= 0);
 
-	CHKRet(relpSessConstruct(&pThis, pSrv->pEngine, RELP_SRV_CONN, pSrv));
+	CHKRet(relpSessConstruct(&pThis, pSrv->pEngine, RELP_SRV_CONN, pSrv, pSrv->pUsr));
+
 	CHKRet(relpTcpAcceptConnReq(&pThis->pTcp, sock, pSrv));
 	CHKRet(relpSessSetMaxDataSize(pThis, pSrv->maxDataSize));
 
@@ -235,6 +268,7 @@ relpSessRcvData(relpSess_t *pThis)
 	CHKRet(relpTcpRcv(pThis->pTcp, rcvBuf, &lenBuf));
 
 	if(lenBuf == 0) {
+		callOnErr(pThis, "server closed relp session, session broken", RELP_RET_SESSION_BROKEN);
 		pThis->pEngine->dbgprint("server closed relp session %p, session broken\n", (void*)pThis);
 		/* even though we had a "normal" close, it is unexpected at this
 		 * stage. Consequently, we consider the session to be broken, because
@@ -242,8 +276,9 @@ relpSessRcvData(relpSess_t *pThis)
 		 */
 		pThis->sessState = eRelpSessState_BROKEN;
 		ABORT_FINALIZE(RELP_RET_SESSION_BROKEN);
-	} else if ((int) lenBuf == -1) { /* I don't know why we need to cast to int, but we must... */
+	} else if ((int) lenBuf == -1) {
 		if(errno != EAGAIN) {
+			callOnErr(pThis, "error when receiving data, session broken", RELP_RET_SESSION_BROKEN);
 			pThis->pEngine->dbgprint("errno %d during relp session %p, session broken\n",
 				errno, (void*)pThis);
 			pThis->sessState = eRelpSessState_BROKEN;
@@ -284,6 +319,7 @@ relpSessSendResponse(relpSess_t *pThis, relpTxnr_t txnr, unsigned char *pData, s
 finalize_it:
 	if(iRet != RELP_RET_OK) {
 		if(iRet == RELP_RET_IO_ERR) {
+			callOnErr(pThis, "io error, session broken", RELP_RET_SESSION_BROKEN);
 			pThis->pEngine->dbgprint("relp session %p is broken, io error\n", (void*)pThis);
 			pThis->sessState = eRelpSessState_BROKEN;
 			}
@@ -531,7 +567,7 @@ relpSessWaitState(relpSess_t *const pThis, const relpSessState_t stateExpected,
 		pfd.fd = sock;
 		pfd.events = POLLIN;
 		pThis->pEngine->dbgprint("relpSessWaitRsp waiting for data on "
-			"fd %d, timeout %d\n", sock, timeout);
+			"fd %d, timeout %d, state expected %d\n", sock, timeout, stateExpected);
 		nfds = poll(&pfd, 1, timeout*1000);
 		if(nfds == -1) {
 			if(errno == EINTR) {
@@ -563,6 +599,8 @@ finalize_it:
 		iRet == RELP_RET_SESSION_BROKEN ||
 		relpEngineShouldStop(pThis->pEngine)) {
 		/* the session is broken! */
+		callOnErr(pThis, "error waiting on required session state, session broken",
+			RELP_RET_SESSION_BROKEN);
 		pThis->sessState = eRelpSessState_BROKEN;
 	}
 
@@ -594,6 +632,7 @@ relpSessRawSendCommand(relpSess_t *pThis, unsigned char *pCmd, size_t lenCmd,
 
 	if(iRet == RELP_RET_IO_ERR) {
 		pThis->pEngine->dbgprint("relp session %p flagged as broken, IO error\n", (void*)pThis);
+		callOnErr(pThis, "io error in RawSendCommand, session broken", RELP_RET_SESSION_BROKEN);
 		pThis->sessState = eRelpSessState_BROKEN;
 		ABORT_FINALIZE(RELP_RET_SESSION_BROKEN);
 	}
@@ -757,6 +796,7 @@ relpSessCBrspOpen(relpSess_t *pThis, relpFrame_t *pFrame)
 			pEngine->dbgprint("ignoring unknown server offer '%s'\n", pOffer->szName);
 		}
 	}
+
 	relpSessSetSessState(pThis, eRelpSessState_INIT_RSP_RCVD);
 
 finalize_it:
@@ -782,8 +822,10 @@ relpSessCltConnChkOffers(relpSess_t *pThis)
 		ABORT_FINALIZE(RELP_RET_RQD_FEAT_MISSING);
 
 finalize_it:
-	if(iRet != RELP_RET_OK)
+	if(iRet != RELP_RET_OK) {
+		callOnErr(pThis, "error in CltConnChkOffers, session broken", RELP_RET_SESSION_BROKEN);
 		pThis->sessState = eRelpSessState_BROKEN;
+	}
 
 	LEAVE_RELPFUNC;
 }
@@ -857,9 +899,19 @@ relpSessConnect(relpSess_t *pThis, int protFamily, unsigned char *port, unsigned
 	relpSessSetSessState(pThis, eRelpSessState_READY_TO_SEND); /* indicate session startup */
 
 finalize_it:
-pThis->pEngine->dbgprint("end relpSessConnect, iRet %d\n", iRet);
-	if(pszOffers != NULL)
-		free(pszOffers);
+	free(pszOffers);
+	if(iRet != RELP_RET_OK) {
+		callOnErr(pThis, "error opening connection to remote peer", iRet);
+		if((pThis->pUnackedLstLast != NULL) &&
+		   !strncmp((char*)pThis->pUnackedLstLast->pSendbuf->pData + 9, " open ", 6)) {
+			pThis->pEngine->dbgprint("relpSessConnect remove 'open' from unacked list\n");
+			relpSessUnacked_t *pDel = pThis->pUnackedLstLast;
+			DLL_Del(pDel, pThis->pUnackedLstRoot, pThis->pUnackedLstLast);
+			--pThis->lenUnackedLst;
+			relpSendbufDestruct(&pDel->pSendbuf);
+			free(pDel);
+		}
+	}
 
 	LEAVE_RELPFUNC;
 }
