@@ -658,14 +658,17 @@ relpTcpDestruct(relpTcp_t **ppThis)
 	RELPOBJ_assert(pThis, Tcp);
 
 	if(pThis->sock != -1) {
+		shutdown(pThis->sock, SHUT_RDWR);
 		close(pThis->sock);
 		pThis->sock = -1;
 	}
 
 	if(pThis->socks != NULL) {
 		/* if we have some sockets at this stage, we need to close them */
-		for(i = 1 ; i <= pThis->socks[0] ; ++i)
+		for(i = 1 ; i <= pThis->socks[0] ; ++i) {
+			shutdown(pThis->socks[i], SHUT_RDWR);
 			close(pThis->socks[i]);
+		}
 		free(pThis->socks);
 	}
 
@@ -1108,8 +1111,13 @@ relpTcpTLSSetPrio(relpTcp_t *const pThis)
 #endif /* defined(ENABLE_TLS)*/
 #if defined(ENABLE_TLS_OPENSSL)
 	if (pThis->authmode == eRelpAuthMode_None)
-		strncpy(pristringBuf, "ALL:+COMPLEMENTOFDEFAULT:+ADH:+ECDH:+aNULL" /* :+aNULL:+eNULL */,
+#	if OPENSSL_VERSION_NUMBER >= 0x10100000L
+		strncpy(pristringBuf, "ALL:+COMPLEMENTOFDEFAULT:+ADH:+ECDH:+aNULL@SECLEVEL=0" /* do not use: +eNULL */,
 			sizeof(pristringBuf));
+#	else
+		strncpy(pristringBuf, "ALL:+COMPLEMENTOFDEFAULT:+ADH:+ECDH:+aNULL" /* do not use: +eNULL */,
+			sizeof(pristringBuf));
+#	endif
 	else
 		strncpy(pristringBuf, "DEFAULT", sizeof(pristringBuf));
 
@@ -2860,6 +2868,7 @@ relpTcpSend(relpTcp_t *const pThis, relpOctet_t *pBuf, ssize_t *pLenBuf)
 #endif /* defined(ENABLE_TLS) | defined(ENABLE_TLS_OPENSSL */
 		written = send(pThis->sock, pBuf, *pLenBuf, 0);
 		const int errno_save = errno;
+		pThis->pEngine->dbgprint("relpTcpSend: send data: %.*s\n", (int) *pLenBuf, pBuf);
 		pThis->pEngine->dbgprint("relpTcpSend: sock %d, lenbuf %zd, send returned %d [errno %d]\n",
 			(int)pThis->sock, *pLenBuf, (int) written, errno_save);
 		if(written == -1) {
@@ -3027,7 +3036,7 @@ relpTcpConnect(relpTcp_t *const pThis,
 	struct addrinfo hints;
 	struct addrinfo *reslocal = NULL;
 	struct pollfd pfd;
-	char errmsg[1024];
+	char errmsg[1424];
 	int so_error;
 	socklen_t len = sizeof so_error;
 	int r;
@@ -3067,7 +3076,7 @@ relpTcpConnect(relpTcp_t *const pThis,
 	}
 	if(connect(pThis->sock, res->ai_addr, res->ai_addrlen) == -1) {
 		if(errno != EINPROGRESS) {
-			char errStr[1024];
+			char errStr[1200];
 			_relpEngine_strerror_r(errno, errStr, sizeof(errStr));
 			snprintf(errmsg, sizeof(errmsg), "error connecting: '%s'", errStr);
 			callOnErr(pThis, errmsg, RELP_RET_IO_ERR);
