@@ -1,6 +1,10 @@
-/* A minimal RELP receiver using librelp
+/* A RELP receiver for testing.
  *
  * Copyright 2014 Mathias Nyman
+ * Copyright 2018 Adiscon GmbH
+ *
+ * See getopt() call below for command line options. There is a brief
+ * (buf hopefully sufficient) comment describing what each option does.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,13 +28,51 @@
 #include <string.h>
 #include <limits.h>
 #include <errno.h>
+#include <signal.h>
 #include "librelp.h"
 
-#define TRY(f) if(f != RELP_RET_OK) { fprintf(stderr, "receive.c: FAILURE in '%s'\n", #f); ret = 1; goto done; }
+#define TRY(f) if(f != RELP_RET_OK) { fprintf(stderr, "receive: FAILURE in '%s'\n", #f); ret = 1; goto done; }
 
 static FILE *errFile = NULL;
+static FILE *outFile = NULL;
+static char *pidFileName = NULL;
+
+static int immediate_exit = 0; /* if set to 1, force-exit as soon as possible */
+
 static relpEngine_t *pRelpEngine;
 
+#define USR_MAGIC 0x1234FFee
+struct usrdata { /* used for testing user pointer pass-back */
+	int magic;
+	char *progname;
+};
+struct usrdata *userdata = NULL;
+static void
+hdlr_enable(int sig, void (*hdlr)())
+{
+	struct sigaction sigAct;
+	memset(&sigAct, 0, sizeof (sigAct));
+	sigemptyset(&sigAct.sa_mask);
+	sigAct.sa_handler = hdlr;
+	sigaction(sig, &sigAct, NULL);
+}
+
+void
+terminate(__attribute__((unused)) const int sig)
+{
+	relpEngineSetStop(pRelpEngine);
+}
+
+/* handler to unconditionally exit the code - required for test where
+ * server must "suddenly" abort.
+ */
+void
+do_exit(__attribute__((unused)) const int sig)
+{
+	immediate_exit = 1;
+}
+
+
 static void __attribute__((format(printf, 1, 2)))
 dbgprintf(char *fmt, ...)
 {
@@ -40,7 +82,7 @@ dbgprintf(char *fmt, ...)
 	va_start(ap, fmt);
 	vsnprintf(pszWriteBuf, sizeof(pszWriteBuf), fmt, ap);
 	va_end(ap);
-	fprintf(stderr, "receive.c: %s", pszWriteBuf);
+	fprintf(stderr, "receive: %s", pszWriteBuf);
 	fflush(stderr);
 }
 
@@ -55,11 +97,16 @@ static relpRetVal onSyslogRcv(unsigned char *pHostname __attribute__((unused)),
 	memset(pMsg, '\0', lenMsg+1);
 	memcpy(pMsg, msg, lenMsg);
 
-	printf("%s\n", pMsg);
-	fflush(stdout);
+	fprintf(outFile, "%s\n", pMsg);
+	fflush(outFile);
 
 	free(pMsg);
 
+	if(immediate_exit) {
+		fprintf(stderr, "receive: force-exit %lld by user request\n", (long long) getpid());
+		exit(1);
+	}
+
 	return RELP_RET_OK;
 }
 
@@ -69,9 +116,17 @@ void print_usage(void)
 }
 
 static void
-onErr( __attribute__((unused)) void *pUsr, char *objinfo, char* errmesg, __attribute__((unused)) relpRetVal errcode)
+onErr(void *pUsr, char *objinfo, char* errmesg, __attribute__((unused)) relpRetVal errcode)
 {
-	fprintf(stderr, "receive: error '%s', object '%s'\n", errmesg, objinfo);
+	struct usrdata *pThis = (struct usrdata*) pUsr;
+	if(pUsr != userdata) {
+		fprintf(stderr, "receive: pUsr NOT pointing to usrdata!\n");
+	}
+	if(pThis->magic != USR_MAGIC) {
+		fprintf(stderr, "receive: pUsr magic incorrect in onErr, magic %8.8x "
+			"pUsr %p\n", pThis->magic, (void*) pThis);
+	}
+	fprintf(stderr, "%s: error '%s', object '%s'\n", pThis->progname, errmesg, objinfo);
 	if(errFile != NULL) {
 		fprintf(errFile, "receive: error '%s', object '%s'\n", errmesg, objinfo);
 	}
@@ -99,9 +154,20 @@ onAuthErr( __attribute__((unused)) void *pUsr, char *authinfo,
 static void
 exit_hdlr(void)
 {
+	if(userdata != NULL) {
+		free(userdata->progname);
+		free(userdata);
+	}
 	if(errFile != NULL) {
 		fclose(errFile);
 	}
+	if(outFile != NULL) {
+		fclose(outFile);
+	}
+	if(pidFileName != NULL) {
+		unlink(pidFileName);
+	}
+
 }
 
 int main(int argc, char *argv[]) {
@@ -110,8 +176,6 @@ int main(int argc, char *argv[]) {
 	int option_index = 0;
 	unsigned char *port = NULL;
 	int verbose = 0;
-	char *pidFileName = NULL;
-	char *errFileName = NULL;
 	int protFamily = 2; /* IPv4=2, IPv6=10 */
 	relpSrv_t *pRelpSrv;
 	int bEnableTLS = 0;
@@ -123,6 +187,8 @@ int main(int argc, char *argv[]) {
 	int maxDataSize = 0;
 	int oversizeMode = 0;
 	int ret = 0;
+	int append_outfile = 0;
+	const char* outfile_name = NULL;
 
 	static struct option long_options[] =
 	{
@@ -133,25 +199,34 @@ int main(int argc, char *argv[]) {
 		{"authmode", required_argument, 0, 'a'},
 		{"pidfile", required_argument, 0, 'F'},
 		{"errorfile", required_argument, 0, 'e'},
+		{"outfile", required_argument, 0, 'O'},
+		{"append-outfile", no_argument, 0, 'A'},
 		{0, 0, 0, 0}
 	};
 
 
-	while((c = getopt_long(argc, argv, "a:e:F:m:o:P:p:Tvx:y:z:", long_options, &option_index)) != -1) {
+	while((c = getopt_long(argc, argv, "a:Ae:F:m:o:O:P:p:Tvx:y:z:", long_options, &option_index)) != -1) {
 		switch(c) {
 		case 'a':
 			authMode = optarg;
 			break;
+		case 'A':
+			append_outfile = 1;
+			break;
 		case 'e':
-			errFileName = optarg;
+			if((errFile = fopen((char*) optarg, "w")) == NULL) {
+				perror(optarg);
+				fprintf(stderr, "receive: error opening error file\n");
+				exit(1);
+			}
 			break;
 		case 'v':
 			verbose = 1;
 			break;
-		case 'F':
+		case 'F': /* pid file name */
 			pidFileName = optarg;
 			break;
-		case 'm':
+		case 'm': /* message size */
 			maxDataSize = atoi(optarg);
 			if(maxDataSize < 128) {
 				printf("maxMessageSize tried to set to %d, "
@@ -165,7 +240,10 @@ int main(int argc, char *argv[]) {
 				maxDataSize = INT_MAX;
 			}
 			break;
-		case 'o':
+		case 'O': /* output file */
+			outfile_name = optarg;
+			break;
+		case 'o': /* oversize mode */
 			if(strcmp("truncate", optarg) == 0) {
 				oversizeMode = RELP_OVERSIZE_TRUNCATE;
 			} else if(strcmp("abort", optarg) == 0) {
@@ -202,13 +280,8 @@ int main(int argc, char *argv[]) {
 
 	atexit(exit_hdlr);
 
-	if(errFileName != NULL) {
-		printf("errfile %s\n", errFileName);
-		if((errFile = fopen((char*) errFileName, "w")) == NULL) {
-			perror(errFileName);
-			goto done;
-		}
-		setvbuf(errFile, NULL, _IONBF, 128);
+	if(outFile == NULL) {
+		outFile = stdout;
 	}
 
 	if(port == NULL) {
@@ -237,6 +310,17 @@ int main(int argc, char *argv[]) {
 		}
 	}
 
+	hdlr_enable(SIGTERM, terminate);
+	hdlr_enable(SIGUSR1, do_exit);
+
+	if(outfile_name != NULL) {
+		if((outFile = fopen(outfile_name, append_outfile ? "a" : "w")) == NULL) {
+			perror(outfile_name);
+			fprintf(stderr, "receive: error opening output file\n");
+			exit(1);
+		}
+	}
+
 	TRY(relpEngineConstruct(&pRelpEngine));
 	TRY(relpEngineSetDbgprint(pRelpEngine, verbose ? dbgprintf : NULL));
 	TRY(relpEngineSetEnableCmd(pRelpEngine, (unsigned char*) "syslog", eRelpCmdState_Required));
@@ -251,6 +335,10 @@ int main(int argc, char *argv[]) {
 
 	TRY(relpEngineListnerConstruct(pRelpEngine, &pRelpSrv));
 	TRY(relpSrvSetLstnPort(pRelpSrv, port));
+	userdata = calloc(1, sizeof(struct usrdata));
+	userdata->magic = USR_MAGIC;
+	userdata->progname = strdup("receive");
+	relpSrvSetUsrPtr(pRelpSrv, userdata);
 	if(maxDataSize != 0) {
 		TRY(relpSrvSetMaxDataSize(pRelpSrv, maxDataSize));
 	}
@@ -294,9 +382,17 @@ int main(int argc, char *argv[]) {
 		fclose(fp);
 	}
 
-	TRY(relpEngineRun(pRelpEngine)); /* Abort with ctrl-c */
+	int i = 0;
+	while(relpEngineRun(pRelpEngine) != RELP_RET_OK) {
+		fprintf(stderr, "receive: error starting relp engine, try %d\n", i);
+		++i;
+		if(i >= 10) {
+			fprintf(stderr, "receive: giving up starting relp engine\n");
+			break;
+		}
+		sleep(1);
+	}
 
-	TRY(relpEngineSetStop(pRelpEngine));
 	TRY(relpEngineDestruct(&pRelpEngine));
 
 done:
