@@ -5,7 +5,8 @@
 
 # "config settings" for the testbench
 TB_TIMEOUT_STARTUP=400  # 40 seconds - Solaris sometimes needs this...
-TESTPORT=31514
+export TESTPORT=31514
+export OUTFILE=librelp.out.log
 export valgrind="valgrind --malloc-fill=ff --free-fill=fe --log-fd=1"
 #export OPT_VERBOSE=-v # uncomment for debugging 
 
@@ -14,12 +15,11 @@ export valgrind="valgrind --malloc-fill=ff --free-fill=fe --log-fd=1"
 ######################################################################
 
 # $1 is name of pidfile to wait for
-function wait_process_startup_via_pidfile() {
-	let "i=0"
+wait_process_startup_via_pidfile() {
+	i=0
 	while test ! -f $1 ; do
-		printf "startup wait %s: %s\n" $1 $i
 		sleep .100
-		let "i++"
+		(( i++ ))
 		if test $i -gt $TB_TIMEOUT_STARTUP
 		then
 		   printf "ABORT! Timeout waiting on startup\n"
@@ -30,49 +30,54 @@ function wait_process_startup_via_pidfile() {
 }
 
 # start receiver WITH valgrind, add receiver command line parameters after function name
-function startup_receiver_valgrind() {
+startup_receiver_valgrind() {
 	printf 'Starting Receiver...\n'
-	$valgrind ./receive -p $TESTPORT -F receive.pid $OPT_VERBOSE $* 1>>librelp.out.log &
+	$valgrind ./receive -p $TESTPORT --outfile $OUTFILE -F receive.pid $OPT_VERBOSE $* &
 	export RECEIVE_PID=$!
 	printf "got receive pid $RECEIVE_PID\n"
 	wait_process_startup_via_pidfile receive.pid
-	sleep 1
 	printf 'Receiver running\n'
 }
 
 # start receiver, add receiver command line parameters after function name
-function startup_receiver() {
+startup_receiver() {
 	printf 'Starting Receiver...\n'
-	./receive -p $TESTPORT -F receive.pid $OPT_VERBOSE $* 1>>librelp.out.log &
+	./receive -p $TESTPORT -F receive.pid --outfile $OUTFILE $OPT_VERBOSE $* &
 	export RECEIVE_PID=$!
 	printf "got receive pid $RECEIVE_PID\n"
 	wait_process_startup_via_pidfile receive.pid
-	sleep 1
 	printf 'Receiver running\n'
 }
 
-# stop receiver
-function stop_receiver() {
-	if [ -f receive.pid ]; then
-		kill $(cat receive.pid) &> /dev/null
+stop_receiver() {
+	if [ "$RECEIVE_PID" == "" ]; then
+		printf 'oops - receiver pid not found in stop_receiver\n'
+		return
 	fi
-	wait -n 5 $(cat receive.pid) &> /dev/null
-#kill $RECEIVE_PID
-	if [ -f receive.pid ]; then
-		# FORCE
-		kill -9 $(cat receive.pid) &> /dev/null
-	fi
-	sleep 1
+	kill $RECEIVE_PID &> /dev/null
+	wait $RECEIVE_PID
+	printf 'receiver %d stopped\n' $RECEIVE_PID
+}
 
-	printf "receiver stopped\n"
+abort_receiver() {
+	if [ "$RECEIVE_PID" == "" ]; then
+		printf 'oops - receiver pid not found in abort_receiver\n'
+		return
+	fi
+set -x; set +v
+	printf 'stopping receiver %d via abort method\n' $RECEIVE_PID
+	kill -USR1 $RECEIVE_PID # &> /dev/null
+	wait $RECEIVE_PID
+	printf 'receiver %d stopped via abort method\n' $RECEIVE_PID
+set +x; set +v
 }
 
 # $1 is the value to check for
 # $2 (optinal) is the file to check
-function check_output() {
+check_output() {
 	EXPECTED="$1"
 	if [ "$2" == "" ] ; then
-		FILE_TO_CHECK="librelp.out.log"
+		FILE_TO_CHECK="$OUTFILE"
 	else
 		FILE_TO_CHECK="$2"
 	fi
@@ -86,12 +91,27 @@ function check_output() {
 	fi
 }
 
+
+# wait until $TESTPORT is no longer bound, e.g. for session closure
+# TODO: evaluate if this function is really used and, if not,
+# TODO: remove again - 2018-11-19 rgerhards
+wait_testport_available() {
+	while true; do
+		printf 'checking NETSTAT\n'
+		if ! netstat -tp | grep -q $TESTPORT; then
+			break
+		fi
+		sleep 1
+	done
+}
+
+
 # $1 is the value to check for
 # $2 (optinal) is the file to check
-function check_output_only() {
+check_output_only() {
 	EXPECTED="$1"
 	if [ "$2" == "" ] ; then
-		FILE_TO_CHECK="librelp.out.log"
+		FILE_TO_CHECK="$OUTFILE"
 	else
 		FILE_TO_CHECK="$2"
 	fi
@@ -114,25 +134,42 @@ function check_output_only() {
 # find who listens on port:
 # netstat -an | grep $TESTPORT
 # ./CI/solaris-findport.sh $TESTPORT
-function cleanup() {
+cleanup() {
 	if [ "$(uname)" == "SunOS" ] ; then
 		pkill -x receive
 		echo pkill result $?
 	fi
 
 	if [ -f receive.pid ]; then
-		kill -9 `cat receive.pid` &> /dev/null
+		kill -9 $(cat receive.pid) &> /dev/null
 	fi
 
-	rm -f receive.pid librelp.out.log *.err.log error.out.log
+	rm -f -- receive.pid $OUTFILE *.err.log error.out.log
 }
 
 # cleanup at end of regular test run
-function terminate() {
+terminate() {
 	cleanup
 	printf "$0 SUCCESS\n"
 }
 
+# check that the output file contains correct number of messages
+# Works on $OUTFILE
+# TODO: check sequence, so that we do not have duplicates...
+check_msg_count() {
+	printf 'We have %s lines in %s\n' $(wc -l < $OUTFILE) $OUTFILE
+	if ! ./chkseq -s1 -e$NUMMESSAGES -f$OUTFILE -d ; then
+		exit 1
+	fi
+	return
+	lines=$(wc -l < $OUTFILE)
+	if [ "$lines" -ne $NUMMESSAGES ]; then
+		printf 'FAIL: message count not correct for %s\n' $OUTFILE
+		printf 'Have %s lines, expected %d\n' "$lines" $NUMMESSAGES
+		exit 1
+	fi
+}
+
 ######################################################################
 # testbench initialization code - do this LAST here in the file
 ######################################################################
@@ -143,7 +180,8 @@ printf "============================================================\n"
 # on Solaris we still have some issues sometimes. Please keep this
 # informational info inside the framework until this can be totally
 # considered revolved - rgerhards, 2018-04-17
-if [ "$(uname)" == "SunOS" ] ; then
+ps -ef|grep receive
+if [ "$(uname)" == "XXSunOS" ] ; then
 	/usr/ucb/ps awwx
 	echo pgrep
 	pgrep receive
